! -------------------------------------------------------------------------
! Bernese GPS Software Version 5.1
! -------------------------------------------------------------------------

MODULE f_asmode

  IMPLICIT NONE

#include "COM_CPU.inc"

CONTAINS

FUNCTION asmode(mjd,prn)

! -------------------------------------------------------------------------
! Purpose:    This function returns the AS (Anti-Spoofing) mode of the GPS
!             constellation (or a selected GPS satellite).
!
! Author:     S. Schaer
!
! Created:    04-Feb-2009
!
! Changes:    09-Feb-2009 SS: Consider Anti-Spoof testing periods
!             28-Jun-2013 SC: use M_BERN with ONLY
!             30-Nov-2016 RD: Add srName, call cpu_enter and cpu_exit
!             10-Apr-2017 ST: Move IMPLICIT NONE to module scope
!             10-Apr-2017 ST: Add #include "COM_CPU.inc"
!             10-Apr-2017 ST: Add IF(srfDebug) in front of CALL cpu_
!
!
! Copyright:  Astronomical Institute
!             University of Bern
!             Switzerland
! -------------------------------------------------------------------------

  USE m_bern, ONLY: r8b, i4b
  USE m_cpu,  ONLY: cpu_enter, cpu_exit

! List of arguments
! ------------------
! Input:
  REAL(r8b)    :: mjd     ! Time argument (MJD)
                          ! =0: undefined
  INTEGER(i4b) :: prn     ! PRN number
                          ! =0: GPS constellation in general

! Output:
  INTEGER(i4b) :: asmode  ! AS mode
                          ! =0: off
                          ! =1: on

! Local Parameters
! ----------------
  CHARACTER(LEN=6), PARAMETER :: srName = 'asmode'


! DEBUG: entry
! ------------
  IF (srfDebug) CALL cpu_enter(srName)

! Set default AS mode
! -------------------
  asmode = 1

! Return if time argument undefined
! ---------------------------------
  IF (mjd == 0d0) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


! Consider Anti-Spoof testing periods
! -----------------------------------
  IF (mjd > 48856.04167d0 .AND. mjd < 48858.41667d0 .AND. &
      (prn ==  0 .OR. prn ==  2 .OR. prn == 16 .OR. prn == 17 .OR. &
       prn == 20 .OR. prn == 21 .OR. prn == 23 .OR. prn == 24 .OR. &
       prn == 25 .OR. prn == 26 .OR. prn == 28               )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 48884.04167d0 .AND. mjd < 48886.41667d0 .AND. &
      (prn ==  0 .OR. prn ==  2 .OR. prn == 16 .OR. prn == 17 .OR. &
       prn == 20 .OR. prn == 21 .OR. prn == 23 .OR. prn == 24 .OR. &
       prn == 25 .OR. prn == 26 .OR. prn == 28               )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 48891.04167d0 .AND. mjd < 48893.41667d0 .AND. &
      (prn == 0  .OR. prn ==  2 .OR. prn == 16 .OR. prn == 17 .OR. &
       prn == 20 .OR. prn == 21 .OR. prn == 23 .OR. prn == 24 .OR. &
       prn == 25 .OR. prn == 26 .OR. prn == 28               )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 48912.04167d0 .AND. mjd < 48914.41667d0 .AND. &
      (prn ==  0 .OR. prn ==  2 .OR. prn == 14 .OR. prn == 15 .OR. &
       prn == 16 .OR. prn == 17 .OR. prn == 20 .OR. prn == 21 .OR. &
       prn == 23 .OR. prn == 25 .OR. prn == 26 .OR. prn == 28)) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 48959.04167d0 .AND. mjd < 48963.41667d0 .AND. &
      (prn ==  0 .OR. prn == 14 .OR. prn == 15 .OR. prn == 16 .OR. &
       prn == 17 .OR. prn == 19 .OR. prn == 20 .OR. prn == 21 .OR. &
       prn == 23 .OR. prn == 24 .OR. prn == 25 .OR. prn == 26 .OR. &
       prn == 28 .OR. prn ==  0                              )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 49044.50000d0 .AND. mjd < 49044.91667d0 .AND. &
      (prn ==  0 .OR. prn ==  1 .OR. prn ==  2 .OR. prn == 14 .OR. &
       prn == 15 .OR. prn == 17 .OR. prn == 19 .OR. prn == 20 .OR. &
       prn == 21 .OR. prn == 23 .OR. prn == 25 .OR. prn == 26 .OR. &
       prn == 27 .OR. prn == 28 .OR. prn == 29               )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


  IF (mjd > 49044.91667d0 .AND. mjd < 49046.08333d0 .AND. &
      (prn ==  0 .OR. prn ==  1 .OR. prn ==  2 .OR. prn == 14 .OR. &
       prn == 15 .OR. prn == 17 .OR. prn == 19 .OR. prn == 20 .OR. &
       prn == 21 .OR. prn == 23 .OR. prn == 25 .OR. prn == 26 .OR. &
       prn == 27 .OR. prn == 28 .OR. prn == 29               )) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF


! Consider non-AS satellites
! --------------------------
!!  IF (prn == 99 .AND. &
!!      mjd > 50480.99999d0 .AND. mjd < 50503.00000d0) THEN
!!    asmode = 0
!!
!!! DEBUG: exit
!!! -----------
!!    IF (srfDebug) CALL cpu_exit(srName)
!!
!!    RETURN
!!  ENDIF

  IF (mjd >= 50503.00000d0) THEN

! DEBUG: exit
! -----------
    IF (srfDebug) CALL cpu_exit(srName)

    RETURN
  ENDIF

! Check whether time argument in a AS-free period
! -----------------------------------------------
  IF ((mjd >           0d0 .AND. mjd < 49383.00000d0) .OR. &
      (mjd > 49826.87499d0 .AND. mjd < 49847.83334d0) .OR. &
      (mjd > 49886.99999d0 .AND. mjd < 49909.00002d0) .OR. &
      (mjd > 49999.99999d0 .AND. mjd < 50022.00001d0) .OR. &
      (mjd > 50480.99999d0 .AND. mjd < 50503.00000d0)) THEN
    asmode = 0
  ENDIF


! DEBUG: exit
! -----------
  IF (srfDebug) CALL cpu_exit(srName)

  RETURN

END FUNCTION asmode

END MODULE f_ASMODE

